// FM-АМ приемник ИШИМ-003 от RX9CDR с настройкой двумя энкодерами
/*
 D2  Энкодер УКВ
 D3  Энкодер УКВ
 D4  Энкодер КВ
 D5  Энкодер КВ
 D6  LCD D7
 D7  LCD D6
 D8  LCD D5
 D9  LCD D4
 D10 LCD Enable
 D11 LCD RW
 D12 LCD RS
 A0  Переключатель диапазонов галетный
 A2  Переключатель шага настройки АМ
 A4  SDA
 A5  SCL
 ----------------------------------------------------------------
 I2C адрес библиотеки "si5351a3.h" 0x60
 I2C адрес библиотеки <TEA5767Radio.h> 0x60
 Как эти модули между собой уживаются на одном адресе не знаю, но всё работает.
 Диапазоны переключаются галетником на 5 положений
 Выход НЧ переключается тем же галетником. Смотри схему.
 */

#include <LiquidCrystalRus.h>					// Библиотека для работы дисплея 1601
#include "Wire.h"								// Библиотека для работы с аппаратными средствами I2C
#include <iarduino_Encoder_tmr.h>         		// Библиотека для работы с энкодерами через аппаратный таймер
#include "si5351a3.h"							// Библиотека для работы с Si5351
#include <TEA5767Radio.h>						// Библиотека для работы с TEA5767
LiquidCrystalRus lcd(12, 11, 10, 9, 8, 7, 6); 	// Однострочный дисплей 16х1 подключаем
iarduino_Encoder_tmr enc_1(2,3);                // Первый энкодер подключаем
iarduino_Encoder_tmr enc_2(4,5);                // Второй энкодер подключаем
TEA5767Radio radio = TEA5767Radio();			// ФМ модуль TEA5767 подключаем
unsigned long bfoAM = 45000000L; 				// Промежуточная частота АМ
unsigned long vfoAM = 10000000L; 				// Стартовая частота АМ
unsigned long stepAM = 100000L;					// Стартовый шаг гц 10кГц
unsigned long LO = 500000L;						// Нижний предел 500кгц
unsigned long HI = 20000000L;					// Верхний предел 20МГц
int buttonstate = 0; 							// Состояние кнопки шага (не нажата)
int Station;									// Переменная для номера FM станции
word channel_raw; 								// Переменная для переключателя диапазонов
int  channel; 									// Переменная для переключателя диапазонов
int  channel_old = 11; 							// Переменная для переключателя диапазонов
byte thousands, tenthousands, hundredthousands, millions, tenmillions ;

void setup() 
{
  Wire.begin();
  Serial.begin(9600);
  lcd.begin(16, 2);									// Подключаем как двухстрочный дисплей, но
  lcd.setDRAMModel(LCD_DRAM_WH1601); 				// конфигурируем как однострочный дисплей 16х1
  enc_1.begin();                               		// Инициируем работу 1 энкодера
  enc_2.begin();                               		// Инициируем работу 2 энкодера
  digitalWrite(2,HIGH);								// резисторы на энкодеры
  digitalWrite(3,HIGH);								// резисторы на энкодеры
  digitalWrite(4,HIGH);								// резисторы на энкодеры
  digitalWrite(5,HIGH);								// резисторы на энкодеры
  pinMode(A2, INPUT); 								// Обьявляем A2 входом кнопки шага
  digitalWrite(A2, HIGH); 							// резистор на кнопку шага
  pinMode(A0, INPUT); 								// Обьявляем А0 входом галетного переключателя
  si5351aXtalCp(_8pF);								// Инициируем генератор
  setFrequencyAM(vfoAM);							// Запускаем генератор
}

void loop() 
{
  channel_raw = analogRead(A0); 					// считываем положение галетника
  channel_raw = channel_raw / 208; 					// Разбиваем его на 5 положений
  channel = channel_raw; 							// Значение равно считанному положению
  if (channel != channel_old)						// Если крутим галетный переключатель
  {lcd.setCursor(0, 0);	
  lcd.print("                "); 					// Очищаем экран
  ChangeBand(); }									// Устанавливаем частоту в зависимости от положения галетника
  if (channel == 0 || channel == 1 || channel == 2 || channel == 3)		// Если на галетнике АМ диапазоны
  { tunAM(); }										// запускаем АМ приемник
  else if (channel == 4)							// Если ФМ диапазон
  {   tunFM();  }									// Запускаем ФМ приемник
  channel_old = channel;							// Запоминаем текущее состояние
}

void ChangeBand() {
  if      (channel == 0)  { LO = 500000L;   HI = 5000000L;  vfoAM = 3950000L;  }
  else if (channel == 1)  { LO = 5000000L;  HI = 10000000L; vfoAM = 9300000L;  }
  else if (channel == 2)  { LO = 10000000L; HI = 15000000L; vfoAM = 13500000L; }
  else if (channel == 3)  { LO = 15000000L; HI = 20000000L; vfoAM = 17500000L; }
  else if (channel == 4)  {	Station = 2;									   }
}

void tunAM() 
{
	int b=enc_2.read();                         	// Читаем состояние 2 энкодера в переменную b
    if(b==encRIGHT)  {vfoAM = vfoAM + stepAM;		// Если крутим вправо - увеличиваем частоту
	if (vfoAM >= HI) {vfoAM = HI;} 					// Верхний предел диапазона - останавливаемся
	} 
    if(b==encLEFT)  {vfoAM = vfoAM - stepAM; 		// Если крутим влево - уменьшаем частоту
	if (vfoAM <= LO) {vfoAM = LO;} 					// Нижний предел диапазона - останавливаемся
	}
	setFrequencyAM(vfoAM);							// Устанавливаем частоту в гетеродине
    showFreqAM();  									// Показываем частоту
	buttonstate = digitalRead(A2); 					// Проверяем состояние кнопки шага
    if (buttonstate == LOW) {incrementAM();}; 		// Если кнопка нажата то меняем шаг
}

void tunFM()
{
	int a=enc_1.read();                         	// Читаем состояние 1 энкодера в переменную b
    if (a==encRIGHT) {Station = Station + 1;} 		// Крутим энкодер вправо - вперед по списку
    if (a==encLEFT)  {Station = Station - 1;} 		// Крутим энкодер влево - назад по списку
	setStationFM (Station);							// Настраиваемся на новую станцию
	if (Station >34) {Station = 1;}					// Доходим до конца списка и возвращаемся в начало
    if (Station <1)  {Station = 34;}				// Доходим до начала списка и идем в конец
}

void incrementAM() 
{
  if      (stepAM == 1000L)  {stepAM = 5000L;}
  else if (stepAM == 5000L)  {stepAM = 10000L;}
  else if (stepAM == 10000L) {stepAM = 100000L;}
  else    {stepAM = 1000L;}
  	lcd.setCursor(0, 0);	lcd.print("                "); 				// Затираем всё
	lcd.setCursor(0, 0);	lcd.print("   STEP ");	lcd.print(stepAM);	// Показываем шаг настройки
    delay(1500); 														// Задержка между переключениями.
  	lcd.setCursor(0, 0);	lcd.print("                "); 				// Затираем всё
};

void showFreqAM() 
{
  tenmillions = ((vfoAM / 10000000) % 10);
  millions = ((vfoAM / 1000000) % 10);
  hundredthousands = ((vfoAM / 100000) % 10);
  tenthousands = ((vfoAM / 10000) % 10);
  thousands = ((vfoAM / 1000) % 10);
  lcd.setCursor(4, 0);
  lcd.print(tenmillions); 							// Выводим на дисплей десятки мегагерц
  lcd.print(millions); 								// Выводим на дисплей мегагерцы
  lcd.print("."); 									// Точка отделяет мегагерцы от килогерцев
  lcd.print(hundredthousands); 						// Выводим на дисплей сотни килогерц
  lcd.print(tenthousands); 							// Выводим на дисплей десятки килогерц
  lcd.print(thousands); 							// Выводим на дисплей килогерцы
  lcd.print(" MHz");  								// Выводим на дисплей " МHz"
}

void setStationFM (int Station) 
{
	lcd.setCursor(0, 0);
	switch (Station) 
	{
	// Раскладка станций для города Екатеринбург.
	case 1:		radio.setFrequency(90.2);	lcd.print("     RADIO SK   ");	break;
	case 2:		radio.setFrequency(87.6);	lcd.print("   RADIO ZVEZDA ");	break;
	case 3:		radio.setFrequency(88.3);	lcd.print("     HIT FM     ");	break;
	case 4:		radio.setFrequency(88.8);	lcd.print("    SILVER FM   ");	break;
	case 5:		radio.setFrequency(89.2);   lcd.print("   DETSK RADIO  ");	break;
	case 6:		radio.setFrequency(89.6);   lcd.print("    ENERGY FM   ");	break;
	case 7:		radio.setFrequency(90.8);   lcd.print("  RADIO MAXIMUM ");	break;
	case 8:		radio.setFrequency(91.4);   lcd.print("   ECHO MOSKVY  ");	break;
	case 9:		radio.setFrequency(91.9);   lcd.print("   NAROD VOLNA  ");	break;
	case 10:	radio.setFrequency(92.3);   lcd.print("   KOMSOMOL PR  ");	break;
	case 11:	radio.setFrequency(92.7);   lcd.print("     SPORT FM   ");	break;
	case 12:	radio.setFrequency(93.7);   lcd.print("    RADIO VERA  ");	break;
	case 13:	radio.setFrequency(94.8);   lcd.print("   NASHE RADIO  ");	break;
	case 14:	radio.setFrequency(95.5);   lcd.print("   RADIO ROSSII ");	break;
	case 15:	radio.setFrequency(95.9);  	lcd.print("    ROMANTIKA   ");	break;
	case 16:	radio.setFrequency(96.3); 	lcd.print("     VESTI FM   ");	break;
	case 17:	radio.setFrequency(98.5);   lcd.print("    LOVE RADIO  ");	break;
	case 18:	radio.setFrequency(98.9);   lcd.print("   DOROGNOE RAD ");	break;
	case 19:	radio.setFrequency(99.4);   lcd.print("   BUSINESS FM  ");	break;
	case 20:	radio.setFrequency(100);    lcd.print("     RETRO FM   ");	break;
	case 21:	radio.setFrequency(100.4);  lcd.print("   RADIO PILOT  ");	break;
	case 22:	radio.setFrequency(100.8);  lcd.print("   RADIO MAYAK  ");	break;
	case 23:	radio.setFrequency(101.2);  lcd.print("   EUROPA PLUS  ");	break;
	case 24:	radio.setFrequency(102);    lcd.print("     YUMOR FM   ");	break;
	case 25:	radio.setFrequency(102.5);  lcd.print("      JAM FM    ");	break;
	case 26:	radio.setFrequency(103.2);  lcd.print("     SHANSON    ");	break;
	case 27:	radio.setFrequency(103.7);  lcd.print("     RADIO C    ");	break;
	case 28:	radio.setFrequency(104.1);  lcd.print("   RADIO DACHA  ");	break;
	case 29:	radio.setFrequency(104.5);  lcd.print("   ROCK ARSENAL ");	break;
	case 30:	radio.setFrequency(105);    lcd.print("     AVTORADIO  ");	break;
	case 31:	radio.setFrequency(105.7);  lcd.print("     RUS RADIO  ");	break;
	case 32:	radio.setFrequency(106.2);  lcd.print("      RADIOLA   ");	break;
	case 33:	radio.setFrequency(107);    lcd.print("    SPUTNIK FM  ");	break;
	case 34:	radio.setFrequency(107.6);  lcd.print("     GOROD FM   ");	break;
	} 
}
 
void setFrequencyAM(long freq)
{
  si5351aSetFrequency(bfoAM + vfoAM,_2mA);	// Запускаем генератор на нужной частоте с нужным уровнем
}

